using Autofac;
using FitConnect.Encryption;
using FitConnect.Models;
using FluentAssertions;
using MockContainer;
using NUnit.Framework;

namespace BasicUnitTest;

[TestFixture]
public class SecurityEventTokenTests {
    [SetUp]
    public void Setup() {
        var container = Container.Create();
        _encryption = new FitEncryption(container.Resolve<KeySet>(), null);
    }

    private const string rejectSubmission =
        SecurityEventToken.RejectSubmissionSchema;

    private FitEncryption _encryption = null!;

    private const string acceptSubmission =
        SecurityEventToken.AcceptSubmissionSchema;


    [Test]
    public void DecodeSubmitSubmissionEvent() {
        const string sec = @"{
  ""$schema"": ""https://schema.fitko.de/fit-connect/set-payload/1.0.0/set-payload.schema.json"",
  ""jti"": ""25d2eb77-458d-4c9d-991c-6428c4651646"",
  ""iss"": ""https://submission-api-dev.fit-connect.fitko.dev"",
  ""iat"": 1622796532,
  ""sub"": ""submission:02bf1d9f-282d-4abf-810a-c4104baf0afe"",
  ""txn"": ""case:452b5ee6-35df-441a-bd39-6141723cf914"",
  ""events"": {
    ""https://schema.fitko.de/fit-connect/events/submit-submission"": {
      ""authenticationTags"": {
        ""metadata"": ""XFBoMYUZodetZdvTiFvSkQ"",
        ""data"": ""UCGiqJxhBI3IFVdPalHHvA"",
        ""attachments"": {
          ""0b799252-deb9-42b0-98d3-c50d24bbafe0"": ""rT99rwrBTbTI7IJM8fU3El"",
          ""25abf553-0e53-43b9-a14a-1581b32a9ee5"": ""i7226HEB7IchCxNuh7lCiu"",
          ""046a9fa5-bed6-494b-aab6-d41056c6db79"": ""d48LxeolRdtFF4nzQibeYO""
        }
      }
    }
  }
}";

        var securityEventToken = SecurityEventToken.FromJson(sec);
        securityEventToken?.EventType.Should().Be(EventType.Submit);
        securityEventToken?.Events?.SubmitSubmissionEvent?.Should().NotBeNull();
        securityEventToken?.Events?.SubmitSubmissionEvent?.AuthenticationTags?.Data.Should()
            .Be("UCGiqJxhBI3IFVdPalHHvA");
    }

    // [Test]
    // public void CreateJwt_AcceptSubmission() {
    //     var token = _encryption.CreateAcceptSecurityEventToken(new SubmissionForPickupDto {
    //         Id = Guid.NewGuid().ToString(), CaseId = Guid.NewGuid().ToString(),
    //         DestinationId = Guid.NewGuid().ToString()
    //     }).Result;
    //     Console.WriteLine(token);
    //     var decoded = new SecurityEventToken(token);
    //     decoded.Events?.Type.Should()
    //         .Be(acceptSubmission);
    //     decoded.EventType.Should().Be(EventType.Accept);
    // }

    // [Test]
    // public void CreateJwt_Reject_WithEncryptionIssue() {
    //     var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         new[] { new Problems(Problems.ProblemTypeEnum.EncryptionIssue, "UnitTest") }
    //     ).Result;
    //     Console.WriteLine(token);
    //     var decoded = new SecurityEventToken(token);
    //     decoded.Events?.Type.Should()
    //         .Be(rejectSubmission);
    //     decoded.EventType.Should().Be(EventType.Reject);
    // }
    //
    // [Test]
    // public void CreateJwt_Reject_WithMissingSchema() {
    //     var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         new[] { new Problems(Problems.ProblemTypeEnum.MissingSchema, "UnitTest") }
    //     ).Result;
    //     Console.WriteLine(token);
    //     var decoded = new SecurityEventToken(token);
    //     decoded.Events?.Type.Should()
    //         .Be(rejectSubmission);
    //
    //     decoded.EventType.Should().Be(EventType.Reject);
    // }
    //
    // [Test]
    // public void CreateJwt_Reject_WithSchemaViolation() {
    //     var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         new[] { new Problems(Problems.ProblemTypeEnum.SchemaViolation, "UnitTest") }
    //     ).Result;
    //     Console.WriteLine(token);
    //     var decoded = new SecurityEventToken(token);
    //     decoded.Events?.Type.Should()
    //         .Be(rejectSubmission);
    //     decoded.EventType.Should().Be(EventType.Reject);
    // }
    //
    // [Test]
    // public void CreateJwt_Reject_WithSyntaxViolation() {
    //     var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         new[] { new Problems(Problems.ProblemTypeEnum.SyntaxViolation, "UnitTest") }
    //     ).Result;
    //     Console.WriteLine(token);
    //
    //     var decoded = new SecurityEventToken(token);
    //     decoded.Events?.Type.Should()
    //         .Be(rejectSubmission);
    //     decoded.EventType.Should().Be(EventType.Reject);
    // }
    //
    // [Test]
    // public void CreateJwt_Reject_WithUnsupportedSchema() {
    //     var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         new[] { new Problems(Problems.ProblemTypeEnum.UnsupportedDataSchema, "UnitTest") }
    //     ).Result;
    //     Console.WriteLine(token);
    //     var decoded = new SecurityEventToken(token);
    //     decoded.Events?.Type.Should()
    //         .Be(rejectSubmission);
    //     decoded.EventType.Should().Be(EventType.Reject);
    // }
    //
    // [Test]
    // public void CreateJwt_Reject_WithIncorrectAuthenticationTag() {
    //     var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(),
    //         new[] { new Problems(Problems.ProblemTypeEnum.IncorrectAuthenticationTag, "UnitTest") }
    //     ).Result;
    //     Console.WriteLine(token);
    //     var decoded = new SecurityEventToken(token);
    //     decoded.Events?.Type.Should()
    //         .Be(rejectSubmission);
    //     decoded.EventType.Should().Be(EventType.Reject);
    // }
    //
    // [Test]
    // public void CreateJwt_WithEncryptionKey_ShouldWork() {
    //     _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(), Guid.NewGuid().ToString(),
    //         Guid.NewGuid().ToString(), "DummyEvent", null).Wait();
    // }
    //
    // [Test]
    // public void CreateJwt_WithEncryptionKey_ShouldThrowAnException() {
    //     var ae = Assert.Throws<AggregateException>(() =>
    //         new FitEncryption(_encryption.PrivateKeyDecryption,
    //                 _encryption.PublicKeyEncryption, // using wrong key
    //                 null) {
    //                 PublicKeyEncryption = _encryption.PublicKeyEncryption,
    //                 PublicKeySignatureVerification = _encryption.PublicKeySignatureVerification
    //             }
    //             .CreateSecurityEventToken(Guid.NewGuid().ToString(), Guid.NewGuid().ToString(),
    //                 Guid.NewGuid().ToString(), "DummyEvent", null).Wait()
    //     );
    //     Assert.AreEqual(
    //         typeof(InvalidOperationException), ae!.InnerException!.GetType(),
    //         "There was no InvalidOperationException.");
    // }
}
